import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
    description = 'Replaces any character that is not one of acgtACGTnN with an N',
    usage = 'fastaq acgtn_only [options] <infile> <outfile>')
    parser.add_argument('infile', help='Name of input file')
    parser.add_argument('outfile', help='Name of output file')
    options = parser.parse_args()
    tasks.acgtn_only(options.infile, options.outfile)




import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = description,
        usage = 'fastaq merge [options] <infile> <outfile>')
    parser.add_argument('infile', help='Name of input file')
    parser.add_argument('outfile', help='Name of output file')
    parser.add_argument('-n', '--name', help='Name of sequence in output file [%(default)s]', default='union')
    options = parser.parse_args()
    tasks.merge_to_one_seq(
        options.infile,
        options.outfile,
        seqname=options.name
    )



import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = description,
        usage = 'fastaq get_seq_flanking_gaps [options] <infile> <outfile>')
    parser.add_argument('--left', type=int, help='Number of bases to get to left of gap [%(default)s]', default=25, metavar='INT')
    parser.add_argument('--right', type=int, help='Number of bases to get to right of gap [%(default)s]', default=25, metavar='INT')
    parser.add_argument('infile', help='Name of input file')
    parser.add_argument('outfile', help='Name of output file')
    options = parser.parse_args()
    tasks.get_seqs_flanking_gaps(options.infile, options.outfile, options.left, options.right)



#!/usr/bin/env python3

import sys
import os
import filecmp
import unittest
from pyfastaq import utils

modules_dir = os.path.dirname(os.path.abspath(utils.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

class TestUtils(unittest.TestCase):
    def test_write_and_read(self):
        '''open_file_write() and open_file_read() should do the right thing depending gzipped or not'''
        for filename in ['utils.tmp', 'utils.tmp.gz', 'utils.tmp.bgz']:
            f = utils.open_file_write(filename)
            for i in range(3):
                print(i, file=f)
            utils.close(f)

            counter = 0

            f = utils.open_file_read(filename)
            for line in f:
                self.assertEqual(counter, int(line.strip()))
                counter += 1
            utils.close(f)

            os.unlink(filename)

        f = utils.open_file_read('-')
        self.assertEqual(sys.stdin, f)
        f = utils.open_file_write('-')
        self.assertEqual(sys.stdout, f)

    def test_raise_exception(self):
        '''open_file_write() and open_file_read() should raise an exception when can't do the opening'''
        with self.assertRaises(utils.Error):
            utils.open_file_read('this_file_is_not_here_so_throw_error')
        with self.assertRaises(utils.Error):
            utils.open_file_read('this_file_is_not_here_so_throw_error.gz')
        with self.assertRaises(utils.Error):
            utils.open_file_read(os.path.join(data_dir, 'utils_test_not_really_zipped.gz'))

        with self.assertRaises(utils.Error):
            utils.open_file_write(os.path.join('not_a_directory', 'this_file_is_not_here_so_throw_error'))
        with self.assertRaises(utils.Error):
            utils.open_file_write(os.path.join('not_a_directory', 'this_file_is_not_here_so_throw_error.gz'))

    def test_file_transpose(self):
        '''Test that file_transpose() does what it should'''
        infile = os.path.join(data_dir, 'utils_test_file_transpose.txt')
        tmp_out = 'utils_test_file_transpose.tmp'
        correct_file = os.path.join(data_dir, 'utils_test_file_transposed.txt')
        utils.file_transpose(infile, tmp_out)
        self.assertTrue(filecmp.cmp(tmp_out, correct_file))
        os.unlink(tmp_out)

    def test_system_call(self):
        '''Test that system call appears to work and die as it should'''
        test_file = os.path.join(data_dir, 'utils_test_system_call.txt')
        tmp_out = 'utils_test_syscall.tmp'
        utils.syscall('cat ' + test_file + ' > ' + tmp_out)
        self.assertTrue(filecmp.cmp(tmp_out, test_file))
        os.unlink(tmp_out)

        with self.assertRaises(utils.Error):
            utils.syscall('thisisveryunlikelytoebarealcommandandshouldthrowerror')

        utils.syscall('echo "this is not the right string" > ' + tmp_out)
        self.assertFalse(filecmp.cmp(tmp_out, test_file))
        os.unlink(tmp_out)

        s = utils.syscall_get_stdout('echo bingo')
        self.assertListEqual(["bingo"], s)
        with self.assertRaises(utils.Error):
            utils.syscall_get_stdout('thisisveryunlikelytoebarealcommandandshouldthrowerror')

if __name__ == '__main__':
    unittest.main()



import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
    description = 'Renames sequences in a file, calling them 1,2,3... etc',
    usage = 'fastaq enumerate_names [options] <infile> <outfile>')
    parser.add_argument('--start_index', type=int, help='Starting number [%(default)s]', default=1)
    parser.add_argument('--rename_file', help='If used, will write a file of old name to new name')
    parser.add_argument('--keep_suffix', action='store_true', help='Use this to keep a /1 or /2 suffix at the end of each name')
    parser.add_argument('--suffix', help='Add the given string to the end of every name', default=None)
    parser.add_argument('infile', help='Name of fasta/q file to be read')
    parser.add_argument('outfile', help='Name of output fasta/q file')
    options = parser.parse_args()
    tasks.enumerate_names(options.infile,
        options.outfile,
        start_index=options.start_index,
        keep_illumina_suffix=options.keep_suffix,
        rename_file=options.rename_file,
        suffix=options.suffix)



import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = 'Prints the number of sequences in input file to stdout',
        usage = 'fastaq count_sequences <infile>')
    parser.add_argument('infile', help='Name of input file')
    options = parser.parse_args()
    print(tasks.count_sequences(options.infile))



import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = 'Makes a multi-FASTA file of random sequences, all of the same length. Each base has equal chance of being A,C,G or T',
        usage = 'fastaq make_random_contigs [options] <contigs> <length> <outfile>')
    parser.add_argument('--first_number', type=int, help='If numbering the sequences, the first sequence gets this number [%(default)s]', default=1)
    parser.add_argument('--name_by_letters', action='store_true', help='Name the contigs A,B,C,... will start at A again if you get to Z')
    parser.add_argument('--prefix', help='Prefix to add to start of every sequence name', default='')
    parser.add_argument('--seed', type=int, help='Seed for random number generator. Default is to use python\'s default', default=None)
    parser.add_argument('contigs', type=int, help='Nunber of contigs to make')
    parser.add_argument('length', type=int, help='Length of each contig')
    parser.add_argument('outfile', help='Name of output file')
    options = parser.parse_args()
    tasks.make_random_contigs(
        options.contigs,
        options.length,
        options.outfile,
        name_by_letters=options.name_by_letters,
        prefix=options.prefix,
        seed=options.seed,
        first_number=options.first_number
    )



#!/usr/bin/env python3

import os
import unittest
from pyfastaq import caf, utils, sequences

modules_dir = os.path.dirname(os.path.abspath(caf.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

class TestCaf(unittest.TestCase):
    def test_get_next_from_file(self):
        '''Test get_next_from_file()'''

        f_in = utils.open_file_read(os.path.join(data_dir, 'caf_test.caf'))

        c = caf.Caf()
        c.get_next_from_file(f_in)
        read = caf.Caf()
        read.id = 'read1.p1k'
        read.seq = sequences.Fasta(read.id, 'NACGTAN')
        read.seq = read.seq.to_Fastq([4, 24, 42, 43, 40, 30, 8])
        read.insert_min = 2000
        read.insert_max = 4000
        read.ligation = '12345'
        read.clone = 'clone1'
        read.clip_start = 1
        read.clip_end = 5
        self.assertEqual(c, read)

        c.get_next_from_file(f_in)
        read = caf.Caf()
        read.id = 'read2.p1k'
        read.seq = sequences.Fasta(read.id, 'CGACGTT')
        read.seq = read.seq.to_Fastq([9, 9, 40, 41, 42, 42, 4])
        read.insert_min = 2000
        read.insert_max = 4000
        read.ligation = '23456'
        read.clone = 'clone2'
        read.clip_start = None
        read.clip_end = None
        self.assertEqual(c, read)

        utils.close(f_in)


if __name__ == '__main__':
    unittest.main()



import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = description,
        usage = 'fastaq to_fasta [options] <infile> <outfile>')
    parser.add_argument('infile', help='Name of input file. Can be any of FASTA, FASTQ, GFF3, EMBL, GBK, Phylip')
    parser.add_argument('outfile', help='Name of output file')
    parser.add_argument('-l', '--line_length', type=int, help='Number of bases on each sequence line of output file. Set to zero for no linebreaks in sequences [%(default)s]', default=60)
    parser.add_argument('-s', '--strip_after_whitespace', action='store_true', help='Remove everything after first whitespace in every sequence name')
    parser.add_argument('-u', '--check_unique', action='store_true', help='Die if any of the output sequence names are not unique')
    options = parser.parse_args()

    tasks.to_fasta(
        options.infile,
        options.outfile,
        line_length=options.line_length,
        strip_after_first_whitespace=options.strip_after_whitespace,
        check_unique=options.check_unique
    )




import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = description,
        usage = 'fastaq strip_illumina_suffix <infile> <outfile>')
    parser.add_argument('infile', help='Name of input file')
    parser.add_argument('outfile', help='Name of output file')
    options = parser.parse_args()
    tasks.strip_illumina_suffix(options.infile, options.outfile)



import argparse
from pyfastaq import tasks

def run(description):
    parser = argparse.ArgumentParser(
        description = description,
        usage = 'fastaq fasta_to_fastq <fasta in> <qual in> <fastq out>')
    parser.add_argument('fasta', help='Name of input FASTA file', metavar='fasta in')
    parser.add_argument('qual', help='Name of input quality scores file', metavar='qual in')
    parser.add_argument('outfile', help='Name of output FASTQ file', metavar='fastq out')
    options = parser.parse_args()
    tasks.fasta_to_fastq(options.fasta, options.qual, options.outfile)
